#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""All REST API routes and functions"""
from cherrydoor.server import (
    api,
    Resource,
    current_user,
    parser,
    dt,
    inputs,
    db,
    escape,
)

__author__ = "opliko"
__license__ = "MIT"
__version__ = "0.5"
__status__ = "Prototype"

parser.add_argument("time_from", help="start of time range")
parser.add_argument("time_to", help="end of time range")
parser.add_argument("username", help="username of requested user")
parser.add_argument("card", help="MiFare card uid")


class Stats(Resource):
    """
	Usage statistics. Accesible on `/api/stats` and `/api/stats/<string:time_from>/<string:time_to>` endpoint
	"""

    def get(self, time_from=None, time_to=None):
        """
		If http GET is used, the api will return list of logs with range determined by request parameters `time_from` and `time_to` or in url (iso8601 formatted).
		When no parameters are present, this method will retuen logs between 7 days ago and today
		Logs are retrived from `logs` collection and need to contain `timestamp` field with a valid datetime as a value.
		"""
        if not current_user.is_authenticated:
            return {"error": "Not Authenticated"}, 401
        params = parser.parse_args()
        try:
            time_from = inputs.datetime_from_iso8601(escape(time_from))
            time_to = inputs.datetime_from_iso8601(escape(time_to))
        except AttributeError:
            try:
                time_from = inputs.datetime_from_iso8601(escape(params["time_from"]))
            except (AttributeError, KeyError):
                time_from = dt.date.today() - dt.timedelta(days=7)
            try:
                time_to = inputs.datetime_from_iso8601(escape(params["time_to"]))
            except (AttributeError, KeyError):
                time_to = dt.datetime.now()
        try:
            results = [
                result
                for result in db.logs.find(
                    {"timestamp": {"$lt": time_to, "$gte": time_from}},
                    {"card": 0, "_id": 0},
                )
            ]
            return results, 200
        except:
            return [], 404


class Card(Resource):
    """
	Card management and retrival. Accesible on `/api/card/<card>` and `/api/card` endpoints.
	"""

    def __init__(self):
        """
		error messages
		"""
        self.card_error = {
            "error": "no card specified. Pass card uid in url or as `card` in request body"
        }
        self.auth_error = {"error": "Not Authenticated"}

    def check_params(self, card=None, params=None):
        """
		Helper function that returns card and username from params and url - if they are there
		"""
        # if there is no card in url, try to get it from request body
        if not card:
            card = params["card"]
            if not card:
                raise KeyError
        card = escape(card.upper())
        # try to get username from request body
        try:
            username = escape(params["username"])
        except KeyError:
            username = None
        return card, username

    def get(self, card=None):
        """
		Used when HTTP GET request is recieved
		This function checks if the card uid is associated with any user
		UID can be a part of url (/api/card/<uid>) or be passed in request body
		"""
        # check if request was made by a logged-in user
        if not current_user.is_authenticated:
            # return error 401 and explanation if that's not the case
            return self.auth_error, 401
        # get arguments from request body
        params = parser.parse_args()
        # make sure card was passed and is a valid uppercase string
        # additionally, get username if it was passed in request body
        try:
            card, username = self.check_params(card, params)
        except KeyError:
            # if there is no card at all, return a 400 error with an explanation message
            return (self.card_error, 400)
        if username:
            # if username was passed with the request body, only check that user or return 404 if he doesn't exist
            result = db.users.find_one_or_404(
                {"username": username, "cards": card}, {"password": 0, "_id": 0}
            )
        else:
            # if no username was passed, check if any user has this card
            result = db.users.find_one_or_404(
                {"cards": card}, {"password": 0, "_id": 0}
            )
        # return the result and status code 200
        return result, 200

    def post(self, card=None):
        """
		If HTTP POST is used, add the card to a user.
		User is determined by `username` passed in request body.
		If there is no `username` present, current user is modified.
		"""
        # check if request was made by a logged-in user
        if not current_user.is_authenticated:
            # return error 401 and explanation if that's not the case
            return self.auth_error, 401
        # get arguments from request body
        params = parser.parse_args()
        # make sure card was passed and is a valid uppercase string
        # additionally, get username if it was passed in request body
        try:
            card, username = self.check_params(card, params)
        except KeyError:
            # if there is no card at all, return a 400 error with an explanation message
            return (self.card_error, 400)
        if username:
            # if username was passed with the request body, add the card to that user
            db.users.find_one_and_update(
                {"username": username}, {"$push": {"cards": card}}
            )
        else:
            # if no username was passed, add the card to current user
            current_user.add_card(card)
        # return true and 201 status on completion
        return True, 201

    def delete(self, card=None):
        """
		If HTTP DELETE is used, delete the card from database of users.
		User is determined by `username` passed in request body.
		If there is no `username` present, card will be removed from any and all users that it's associated with.
		If `*` is passed as username, returns all users.
		"""
        if not current_user.is_authenticated:
            return self.auth_error, 401
        params = parser.parse_args()
        # make sure card was passed and is a valid uppercase string.
        # additionally, get username if it was passed in request body
        try:
            card, username = self.check_params(card, params)
        except KeyError:
            # if there is no card at all, return a 400 error with an explanation message
            return (self.card_error, 400)
        if username:
            db.users.find_one_and_update(
                {"username": username}, {"$pull": {"cards": card}}
            )
        else:
            db.users.update({}, {"$pull": {"cards": card}})
        return True, 200


class UserAPI(Resource):
    """
	User data management and retrieval. Accesible on `api/user/<username>` and `/api/user` endpoints.
	"""

    def __init__(self):
        """
		error messages
		"""
        self.username_error = {
            "error": "no username specified. Pass username in url or as `username` in request body"
        }
        self.auth_error = {"error": "Not Authenticated"}

    def check_username(self, username, params):
        """
		Helper function for getting and validating username from url or request body
		"""
        if not username:
            params = parser.parse_args()
            username = escape(params["username"])
            if username == None:
                raise KeyError
        username = escape(username)
        return username

    def get(self, username=None):
        """
		If HTTP GET is used, see if user with username passed in url or in request body (as `username`) exists and return their username and cards associated with them.
		When no user with request username exists, return 404 status code
		"""
        if not current_user.is_authenticated:
            return self.auth_error, 401
        params = parser.parse_args()
        try:
            username = self.check_username(username, params)
        except KeyError:
            return (self.username_error, 400)
        if username == "*":
            result = list(db.users.find({}, {"password": 0, "_id": 0}))
        else:
            result = db.users.find_one_or_404(
                {"username": username}, {"password": 0, "_id": 0}
            )
        return result, 200

    def post(self, username=None):
        """
		If HTTP POST is used, create a passwordless user (unable to manage the door, able to enter)
		"""
        if not current_user.is_authenticated:
            return self.auth_error, 401
        params = parser.parse_args()
        try:
            username = self.check_username(username, params)
        except KeyError:
            return (self.username_error, 400)
        try:
            card = escape(params["card"].upper())
            if not card:
                raise KeyError
        except KeyError:
            card = ""
        db.users.update_one(
            {"username": username},
            {"$set": {"username": username, "cards": [card]}},
            upsert=True,
        )
        return None, 201

    def delete(self, username=None):
        """
		If HTTP DELETE is used, delete the specified user
		"""
        if not current_user.is_authenticated:
            return self.auth_error, 401
        params = parser.parse_args()
        try:
            username = self.check_username(username, params)
        except KeyError:
            return (self.username_error, 400)
        db.users.delete_one({"username": username})
        return None, 204


api.add_resource(Stats, "/api/stats", "/api/stats/<string:time_from>/<string:time_to>")
api.add_resource(UserAPI, "/api/user/<string:username>", "/api/user")
api.add_resource(Card, "/api/card/<string:card>", "/api/card")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""All websocket routes and functions"""
import functools
from bson import json_util
import json as jsn
from cherrydoor.server import socket, emit, dt, db, current_user, disconnect

__author__ = "opliko"
__license__ = "MIT"
__version__ = "0.5.dev"
__status__ = "Prototype"


def authenticated_only(f):
    @functools.wraps(f)
    def wrapped(*args, **kwargs):
        if not current_user.is_authenticated:
            disconnect()
        else:
            return f(*args, **kwargs)

    return wrapped


@socket.on("stats", namespace="/api")
@authenticated_only
def stats(json={}):
    try:
        time_from = dt.datetime.fromisoformat(json["time_from"].replace("Z", ""))
    except KeyError:
        time_from = dt.datetime.today() - dt.timedelta(days=7)
    try:
        time_to = dt.datetime.fromisoformat(json["time_to"].replace("Z", ""))
    except KeyError:
        time_to = dt.datetime.now()

    results = db.logs.find(
        {"timestamp": {"$lt": time_to, "$gte": time_from}}, {"card": 0, "_id": 0}
    )
    json_results = [jsn.dumps(doc, default=json_util.default) for doc in results]
    emit("stats", json_results, namespace="/api")
    return json_results


@socket.on("user", namespace="/api")
@authenticated_only
def user(json={}):
    try:
        username = json["username"]
        user = db.users.find_one({"username": username}, {"password": 0, "_id": 0})
        if not user:
            raise KeyError
    except KeyError:
        try:
            card = json["card"]
            user = db.users.find_one({"cards": card}, {"password": 0, "_id": 0})
            if not user:
                raise KeyError
        except KeyError:
            return False
    try:
        if json["edit"]:
            db.users.update_one(user, json["changes"])
    except KeyError:
        pass
    emit("user", user)
    return user


@socket.on("users", namespace="/api")
@authenticated_only
def users():
    try:
        users = db.users.find({}, {"password": 0, "_id": 0})
        json_results = [jsn.dumps(doc, default=json_util.default) for doc in users]
    except:
        return False
    emit("users", json_results)
    return json_results


@socket.on("break_times", namespace="/api")
@authenticated_only
def break_times(json=[]):
    if isinstance(json, list) and len(json) != 0 and isinstance(json[0], list):
        if not any(json[0]):
            db.settings.update(
                {"setting": "break_times"},
                {"setting": "break_times", "value": []},
                upsert=True,
            )
            return_breaks = []
        else:
            try:
                breaks = [
                    [
                        dt.datetime.fromisoformat(item[0].replace("Z", "")),
                        dt.datetime.fromisoformat(item[1].replace("Z", "")),
                    ]
                    for item in json
                ]
            except IndexError:
                return None
            db.settings.update(
                {"setting": "break_times"}, {"$set": {"value": breaks}}, upsert=True,
            )
            breaks = [
                [item[0].isoformat() + "Z", item[1].isoformat() + "Z"]
                for item in breaks
            ]
            return_breaks = jsn.dumps(breaks, indent=4, sort_keys=True, default=str)
        emit("break_times", return_breaks)
        return return_breaks
    try:
        breaks = list(db.settings.find_one({"setting": "break_times"})["value"])
        breaks = [
            [item[0].isoformat() + "Z", item[1].isoformat() + "Z"] for item in breaks
        ]
        return_breaks = jsn.dumps(breaks, indent=4, sort_keys=True, default=str)
    except KeyError:
        return None
    emit("break_times", return_breaks)
    return return_breaks



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Main cherydoor module file
Creates app, api, socket and db instances and imports all routes.
"""
# built-in libraries import:
import json
import datetime as dt
from hashlib import sha256, sha384
import base64
from pathlib import Path

# flask-connected imports:
from flask import Flask, escape, url_for
from flask_login import current_user, LoginManager, UserMixin
from flask_restful import Resource, Api, reqparse, inputs, abort
from flask_socketio import SocketIO, emit, disconnect
from flask_talisman import Talisman
from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField, SubmitField, BooleanField
from wtforms.validators import DataRequired

# hashing function import:
from argon2 import PasswordHasher

__author__ = "opliko"
__license__ = "MIT"
__version__ = "0.5"
__status__ = "Prototype"
default_routes = [
    "config.json",
    "/var/cherrydoor/config.json",
    f"{Path.home()}/.config/cherrydoor/config.json",
]
for route in default_routes:
    try:
        # load configuration file from one of the default routes
        with open(route, "r", encoding="utf-8") as f:
            # convert confuguration to a dictionary using json.load()
            config = json.load(f)
            break
    except FileNotFoundError:
        # ignore if config wasn't found
        pass
if config == None:
    raise FileNotFoundError("No config.json found")


class LoginForm(FlaskForm):
    """create fields for login form with labels based on translations form config.json file"""

    username = StringField(
        config["login-translation"]["username"],
        validators=[DataRequired()],
        render_kw={"placeholder": config["login-translation"]["username"]},
    )
    password = PasswordField(
        config["login-translation"]["password"],
        validators=[DataRequired()],
        render_kw={"placeholder": config["login-translation"]["password"]},
    )
    remember = BooleanField(config["login-translation"]["remember-me"])
    submit = SubmitField(config["login-translation"]["log-in"])


# app creation
# create a Flask instance
app = Flask(__name__, template_folder="../templates", static_folder="../static")
# set up a secret key for cookie and session encryption based on config.json
app.config["SECRET_KEY"] = config["secret-key"]
app.config["TEMPLATES_AUTO_RELOAD"] = True
# pymongo connection
# configure database access uri
try:
    if config["mongo"]:
        from flask_pymongo import PyMongo

        app.config[
            "MONGO_URI"
        ] = f"mongodb://{config['mongo']['url']}/{config['mongo']['name']}"
        try:
            # set up PyMongo using credentials from config.json
            db = PyMongo(
                app,
                username=config["mongo"]["username"],
                password=config["mongo"]["password"],
            ).db
        except KeyError:
            # if username or password aren't defined in config, don't use them at all
            db = PyMongo(app).db
        try:
            db.client.server_info()
        except Exception as e:
            print(
                f"Connection to MongoDB failed. Are you sure it's installed and correctly configured? Error: {e}"
            )
except KeyError:
    print("No supported database present in config.json")
# create login_manager for flask_login
login_manager = LoginManager()
# default view (page) used for logging in
login_manager.login_view = "login"
# set a better session protection
login_manager.session_protection = "strong"
# message shown to anonymous users when a page requires being logged in - set in config.json
login_manager.login_message = config["login-translation"]["message"]
# initialize the login_manager
login_manager.init_app(app)

# create an argon2 hasher instance that will be called for all future operations
hasher = PasswordHasher(
    time_cost=4,
    memory_cost=65536,
    parallelism=8,
    hash_len=16,
    salt_len=16,
    encoding="utf-8",
)

# create a restful api instance from flask_restful
api = Api(app)

# create a flask_restful request argument parser instance
parser = reqparse.RequestParser()

# create socketio instance
socket = SocketIO(app)


csp = {
    "default-src": ["'none'"],
    "script-src": ["'self'"],
    "style-src": ["'self'"],
    "font-src": ["'self'"],
    "img-src": ["'self'"],
    "connect-src": ["'self'"],
    "base-uri": ["'none'"],
    "form-action": ["'self'"],
    "require-sri-for": ["scripts", "styles"],
}
try:
    if config["https"]["enabled"]:
        csp.update({"block-all-mixed-content": [], "upgrade-insecure-requests": []})
except KeyError:
    pass
fp = {
    "accelerometer": "'none'",
    "ambient-light-sensor": "'none'",
    "autoplay": "'none'",
    "battery": "'none'",
    "camera": "'self'",
    "display-capture": "'none'",
    "document-domain": "'none'",
    "encrypted-media": "'none'",
    "execution-while-not-rendered": "'self'",
    "execution-while-out-of-viewport": "'self'",
    "fullscreen": "'self'",
    "geolocation": "'none'",
    "gyroscope": "'none'",
    "layout-animations": "'self'",
    "legacy-image-formats": "'self'",
    "magnetometer": "'none'",
    "microphone": "'none'",
    "midi": "'none'",
    "navigation-override": "'none'",
    "oversized-images": "'none'",
    "payment": "'none'",
    "picture-in-picture": "'none'",
    "publickey-credentials": "'self'",
    "speaker": "'self'",
    "sync-xhr": "'self'",
    "usb": "'none'",
    "vr": "'none'",
    "wake-lock": "'none'",
    "xr-spatial-tracking": "'none'",
}
try:
    Talisman(
        app,
        force_https=config["https"]["enabled"],
        force_https_permanent=config["https"]["enabled"],
        session_cookie_secure=config["https"]["enabled"],
        feature_policy=fp,
        content_security_policy=csp,
        content_security_policy_report_uri="/csp-reports",
        strict_transport_security=config["https"]["hsts-enabled"],
        strict_transport_security_preload=config["https"]["hsts-preload"],
        #        content_security_policy_nonce_in=["script-src", "style-src"],
        referrer_policy="no-referrer",
    )
except KeyError:
    Talisman(
        app,
        feature_policy=fp,
        content_security_policy=csp,
        content_security_policy_report_uri="/csp-reports",
    )


class User(UserMixin):
    """
	User class used by flask_login
	"""

    def __init__(self, username):
        self.username = username

    @staticmethod
    def is_authenticated(self):
        """
		Authentication status
		"""
        return True

    @staticmethod
    def is_active(self):
        """
		Shows that the user is logged in
		"""
        return True

    @staticmethod
    def is_anonymous(self):
        """
		A logged in user is not anonymous
		"""
        return False

    def get_id(self):
        """
		Returns the id - in this case username
		"""
        return self.username

    def get_cards(self):
        """
		Returns all mifare card ids associated with the account
		"""
        return db.users.find_one({"username": self.username})["cards"]

    def add_card(self, card):
        """
		adds a mifare card id to user profile
		"""
        db.users.update_one(
            {"username": self.username}, {"$push": {"cards": card}}, upsert=True
        )

    def delete_card(self, card):
        """
		adds a mifare card id from user profile
		"""
        db.users.update_one({"username": self.username}, {"$pull": {"cards": card}})


@login_manager.user_loader
def load_user(username):
    """
	A function for loading users from database by username
	"""
    u = db.users.find_one({"username": username})
    if not u:
        return None
    return User(username=u["username"])


def sri_for(endpoint, **values):
    input = url_for(endpoint, **values)
    input = input.replace(app.static_url_path, app.static_folder)
    hash = sha256()
    with open(input, "rb") as f:
        while True:
            data = f.read(65536)
            if not data:
                break
            hash.update(data)
    hash = hash.digest()
    hash_base64 = base64.b64encode(hash).decode()
    return f"sha256-{hash_base64}"


app.jinja_env.globals["sri_for"] = sri_for

import cherrydoor.server.api
import cherrydoor.server.routes
import cherrydoor.server.websockets



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""All website routes"""
# flak-related imports
from flask import render_template, url_for, request, session, redirect, flash, escape
from flask_login import current_user, login_user, logout_user, login_required

# import VerificationError thrown when password doesn't match the hash
from argon2.exceptions import VerificationError

from cherrydoor.server import app, db, hasher, LoginForm, login_manager, User, json

__author__ = "opliko"
__license__ = "MIT"
__version__ = "0.5"
__status__ = "Prototype"

# dashboard page
@app.route("/")
@login_required
def index():
    return render_template("index.html")


# login page
@app.route("/login", methods=["GET", "POST"])
def login():
    # initiate form
    form = LoginForm()
    # if form wasn't validated, render login page without error indication
    if not form.validate_on_submit():
        return render_template("login.html", form=form, error="")
    try:
        # escape username and password
        username = escape(form.username.data)
        password = escape(form.password.data)
        user = db.users.find_one({"username": username})
        if not user:
            # if no user with selected is found raise VerificationError to return the template with error on inputs
            raise VerificationError
        validate = hasher.verify(
            user["password"].encode("utf-8"), password.encode("utf-8")
        )
        # if argon2 throws VerificationError it means the password doesn't match the hash for this username
        if user["password"] != "" and validate:
            user_obj = User(username=username)
            login_user(user_obj, remember=escape(form.remember.data))
            # if login was succesful - redirect user to the dashboard
            return redirect(url_for("index"))
    except (VerificationError, KeyError):
        pass
    return render_template("login.html", form=form, error=" is-invalid")


@app.route("/register", methods=["POST", "GET"])
@login_required
def register():
    # render the registration form if the request type isn't POST
    if request.method != "POST":
        return render_template("register.html")
    # otherwise check if username and password were passed to this function
    try:
        username = escape(request.form["username"])
        password = escape(request.form["pass"].encode("utf-8"))
    except:
        return "No username or password specified", 400
    # check if an user with this username exists
    existing_user = db.users.find_one({"username": username})
    # if the user exists - return a 401 message with that information
    if existing_user:
        return {"error": "That username already exists!"}, 401
    # if not - register him
    try:
        # if a card id was passed along - create a cards variable with it
        cards = [escape(request.form["card"])]
    except KeyError:
        # if not - make it empty
        cards = []
    # hash the password
    hashpass = hasher.hash(password)
    # insert the user to database
    users.insert({"username": username, "password": hashpass, "cards": cards})
    # log the user in
    user_obj = User(username=username)
    login_user(user_obj, remember=False)
    # redirect user to index
    return redirect(url_for("index"))


@app.route("/csp-reports", methods=["POST"])
def csp():
    try:
        open("csp-logs.json", "a").close()
    except PermissionError:
        return
    with open("csp-logs.json", "r+", encoding="utf-8") as f:

        try:
            logs = json.load(f)
        except json.decoder.JSONDecodeError:
            logs = {"logs": []}
        try:
            logs["logs"].append(json.loads(request.data.decode("utf-8")))
            f.seek(0, 0)
            json.dump(logs, f)
        except:
            return None, 400
    return None, 201


@app.route("/logout")
@login_required
def logout():
    logout_user()
    return redirect(url_for("login"))


@login_manager.unauthorized_handler
def unauthorized():
    return redirect(url_for("login"))



from json import load
from pymongo import MongoClient
from pathlib import Path

__author__ = "opliko"
__license__ = "MIT"
__version__ = "0.5"
__status__ = "Prototype"

default_routes = [
    "config.json",
    "/var/cherrydoor/config.json",
    f"{Path.home()}/.config/cherrydoor/config.json",
]
for route in default_routes:
    try:
        # load configuration file from one of the default routes
        with open(route, "r", encoding="utf-8") as f:
            # convert confuguration to a dictionary using json.load()
            config = load(f)
            break
    except FileNotFoundError:
        # ignore if config wasn't found
        pass
if config == None:
    raise FileNotFoundError("No config.json found")
try:
    if config["mongo"]:

        try:
            # set up PyMongo using credentials from config.json
            db = MongoClient(
                f"mongodb://{config['mongo']['url']}/{config['mongo']['name']}",
                username=config["mongo"]["username"],
                password=config["mongo"]["password"],
            ).cherrydoor
        except KeyError:
            # if username or password aren't defined in config, don't use them at all
            db = MongoClient(
                f"mongodb://{config['mongo']['url']}/{config['mongo']['name']}"
            ).cherrydoor
        try:
            db.client.server_info()
        except Exception as e:
            print(
                f"Connection to MongoDB failed. Are you sure it's installed and correctly configured? Error: {e}"
            )
except KeyError:
    print("No supported database present in config.json")

try:
    if config["interface"]["type"].lower() == "serial":
        import serial

        connectionException = serial.serialutil.SerialException
        interface = serial.Serial()
        try:
            interface.baudrate = int(config["interface"]["baudrate"])
        except (KeyError, AttributeError):
            # default to 115200 if no baudrate is found in config
            interface.baudrate = 115200
        try:
            interface.port = config["interface"]["port"]
        except (KeyError, AttributeError):
            # default to /dev/serial0 (default for RaspberryPi UART)
            interface.port = "/dev/serial0"
        try:

            encoding = config["interface"]["encoding"]
        except KeyError:
            # default to utf-8 if no encoding information found in config
            encoding = "utf-8"

        def read():
            message = interface.readline().decode(encoding)
            return message

        def write(message):
            interface.write(f"{message}\n".encode(encoding))
            interface.flush()

    else:
        raise KeyError("Unknown interface type")
except (KeyError, AttributeError):
    # default to serial if no compatible interface if found in config file, or interface is not a string
    import serial

    connectionException = serial.serialutil.SerialException
    interface = serial.Serial()
    try:
        interface.baudrate = int(config["interface"]["baudrate"])
    except (KeyError, AttributeError):
        # default to 115200 if no baudrate is found in config
        interface.baudrate = 115200
    try:
        interface.port = config["interface"]["port"]
    except (KeyError, AttributeError):
        # default to /dev/serial0 (default for RaspberryPi UART)
        interface.port = "/dev/serial0"
    try:
        encoding = config["interface"]["encoding"]
    except KeyError:
        # default to utf-8 if no encoding information found in config
        encoding = "utf-8"

    def read():
        message = interface.readline().decode(encoding)
        return message

    def write(message):
        interface.write(f"{message}\n".encode(encoding))
        interface.flush()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Run the server"""
from multiprocessing import Process
import argparse
import sys
import os
import json
from subprocess import call
from argon2 import PasswordHasher
from pymongo import MongoClient


def cherrydoor():
    parser = argparse.ArgumentParser(
        prog="cherrydoor", description="Cherrydoor management"
    )
    subparsers = parser.add_subparsers(dest="subcommand")
    install_parser = subparsers.add_parser(
        "install", help="Install some possible requirements"
    )
    install_parser.add_argument(
        "--exit-on-fail",
        description="If any step fails, stop the installer",
        dest="fail",
        action="store_true",
    )
    install_steps_group = install_parser.add_argument_group(
        "steps",
        "installation steps you want to run (if none are selected all will be run)",
    )
    install_steps = {
        "dependencies": "install all dependencies that weren't installed with pip",
        "service": "set up a systemd unit file",
        "config": "create a config file",
        "database": "set up MongoDB user, collections, etc.",
        "user": "create a new administrator user",
    }
    for (step, description) in install_steps.items():
        install_steps_group.add_argument(
            f"--{step}",
            dest="install_steps",
            action="append_const",
            const=step,
            description=description,
        )
        install_steps_group.add_argument(
            f"--no-{step}",
            dest="install_steps_excluded",
            action="append_const",
            const=step,
            description=f"don't {description}",
        )

    start_parser = subparsers.add_parser(
        "start",
        help="Explicitly start the server (this action is preformed if no other argument is passed too)",
    )

    args = parser.parse_args()
    if args.subcommand == "install":
        from cherrydoor.cherrydoor.install import install

        install(args)

    # if start argument was passed or no arguments were used, start the server
    if args.subcommand == "start" or not len(sys.argv) > 1:
        from cherrydoor.server import app, socket, config
        from cherrydoor.interface.commands import Commands

        interface = Commands()
        interface_run = Process(target=interface.start)
        server = Process(
            target=socket.run,
            kwargs={
                "app": app,
                "log_output": True,
                "host": config["host"],
                "port": config["port"],
            },
        )

        def exit():
            interface_run.terminate()
            server.terminate()

        import atexit

        atexit.register(exit)
        interface_run.start()
        server.run()


if __name__ == "__main__":
    cherrydoor()



from getpass import getpass
import sys
import os
import json
from subprocess import call
from argon2 import PasswordHasher
from pymongo import MongoClient
from pathlib import Path


def step_enabled(step, args):
    return step not in args.install_steps_excluded and (
        args.install_steps == None or step in args.install_steps
    )


def install(args):
    if sys.platform == "linux" or 1 == 1:

        if step_enabled("dependencies", args):
            # install MongoDB and some other things if they're not installed
            try:
                call(["cherrydoor-install"], shell=False)
            except (PermissionError, FileNotFoundError):
                print("unable to install dependencies")
                if args.fail:
                    sys.exit(1)
        # generate a configuration based on default config
        if (
            not os.path.exists(f"{Path.home()}/.config/cherrydoor/config.json")
            or "config" in args.install_steps
        ):
            config = {
                "__comment__": "This is a default config for setuptools installation - it shouldn't be used if installed from GitHub",
                "host": "127.0.0.1",
                "port": 5000,
                "mongo": {
                    "url": "localhost:27017",
                    "name": "cherrydoor",
                    "username": "cherrydoor",
                    "password": "test",
                },
                "login-translation": {
                    "username": "Nazwa użytkownika",
                    "password": "Hasło",
                    "remember-me": "Pamiętaj mnie",
                    "log-in": "Zaloguj się",
                    "message": "Musisz się zalogować by uzyskać dostęp do tej strony",
                },
                "secret-key": "\\xd7w7\\x04\\r\\xfc/q\\x1a\\x9b&",
                "https": {
                    "enabled": False,
                    "hsts-enabled": False,
                    "hsts-preload": False,
                },
                "interface": {
                    "type": "serial",
                    "baudrate": 115200,
                    "port": "/dev/serial0",
                    "encoding": "utf-8",
                },
                "manufacturer-code": "18",
            }
        else:
            with open(
                f"{Path.home()}/.config/cherrydoor/config.json", "r", encoding="utf-8"
            ) as f:
                config = json.load(f)
        if step_enabled("config", args):
            # create a random secret key
            config["secret-key"] = os.urandom(24).hex()
        # let user choose a password for the database
        config["mongo"]["password"] = getpass("Wprowadź hasło do bazy danych: ")
        service_config = f"""\
[Unit]
Description=Cherrydoor Service
After=network.target
[Service]
ExecStart={os.path.realpath(__file__)} start
Environment=PYTHONUNBUFFERED=1
Restart=always
Type=simple
User=ubuntu
[Install]
WantedBy=multi-user.target
"""
        try:
            # files configuration
            if not os.path.exists(f"${Path.home()}"):
                os.makedirs(f"{Path.home()}/.config/cherrydoor")
            with open(
                f"{Path.home()}/.config/cherrydoor/config.json", "w", encoding="utf-8"
            ) as f:
                json.dump(config, f, ensure_ascii=False, indent=4)
        except (IOError, PermissionError):
            print(
                f"Nie udało się stworzyć plików w {Path.home()}/.config/cherrydoor. Spróbuj stworzyć ten folder manualnie i nadać mu właściwe uprawnienia",
                file=sys.stderr,
            )
            if args.fail:
                sys.exit(1)
        if step_enabled("service", args):
            try:
                with open(f"{Path.home()}/systemd/user/cherrydoor.service", "w") as f:
                    f.write(service_config)
                    print(
                        f"Plik konfiguracyjny znajduje się w folderze {Path.home()}/.config/cherrydoor"
                    )
            except (IOError, PermissionError):
                print(
                    f"Nie udało się stworzyć pliku usługi pod {Path.home()}/.config/systemd/user/cherrydoor.service - spróbuj uruchomić skrypt z właściwymi uprawnieniami lub stworzyć ten plik manualnie. Zawartość:",
                    file=sys.stderr,
                )
                print(service_config, file=sys.stderr)
                if args.fail:
                    sys.exit(1)
        hasher = PasswordHasher(
            time_cost=4,
            memory_cost=65536,
            parallelism=8,
            hash_len=16,
            salt_len=16,
            encoding="utf-8",
        )
        db = MongoClient(
            f"mongodb://{config['mongo']['url']}/{config['mongo']['name']}"
        )[config["mongo"]["name"]]
        if step_enabled("database", args):
            db.command(
                "createUser",
                config["mongo"]["username"],
                pwd=config["mongo"]["password"],
                roles=[
                    {"role": "readWrite", "db": config["mongo"]["name"]},
                    {"role": "clusterMonitor", "db": "admin"},
                ],
            )
            db.create_collection("users")
            db.create_collection("logs")
            db.create_collection("settings")
            user_indexes = db.users.index_information()
            if "username_index" not in user_indexes.keys():
                db.users.create_index("username", name="username_index", unique=True)
            if "cards_index" not in user_indexes.keys():
                db.user.create_index("cards", name="cards_index", sparse=True)
        if step_enabled("user", args) and input(
            "Czy chcesz stworzć nowego użytkownika-administratora? [y/n]"
        ).lower() in ["y", "yes", "tak", "t"]:
            username = input("Wprowadź nazwę użytkownika: ")
            password = hasher.hash(getpass("Hasło: "))
            db.users.insert({"username": username, "password": password, "cards": []})
        print("Instalacja skończona!")
        try:
            service_call_args = ["systemctl", "--user", "enable", "cherrydoor"]
            call(service_call_args, shell=False)
        except (IOError, PermissionError):
            pass
    else:
        print("Ten system operacyjny nie jest obecnie obsługiwany")

    sys.exit()



import datetime as dt
from time import sleep
from datetimerange import DateTimeRange
from cherrydoor.interface import read, write, db, config, interface, connectionException
from pymongo.errors import OperationFailure

__author__ = "opliko"
__license__ = "MIT"
__version__ = "0.5"
__status__ = "Prototype"


class Commands:
    def __init__(self):
        self.commandFunctions = {"CARD": self.card}
        try:
            self.require_auth = bool(
                db.settings.find_one({"setting": "require_auth"})["value"]
            )
            write(f"NTFY {4 if self.require_auth else 3}")
        except (KeyError, TypeError, OperationFailure):
            self.require_auth = True

    def start(self):
        # message template - a two element list (command and argument)
        message = ["", ""]
        try:
            with interface:
                # continuously read the interface until EXIT is sent
                while message == [] or message[0] != "EXIT":
                    message = read().upper().split()
                    # after a newline, do the specidied command
                    try:
                        self.commandFunctions[message[0]](message[1])
                    except (KeyError, IndexError):
                        pass
        except connectionException:
            sleep(10)
            self.start()

    def card(self, block0):
        try:
            # check if authentication with UID is required, or manufacturer code is fine
            self.require_auth = self.check_auth()
            # send a contol signal for LEDs
            write(f"NTFY {4 if self.require_auth else 3}")
        except:
            # default to requiring authentication
            self.require_auth = True
        if self.require_auth:
            # check if card is associated with an user
            auth = bool(db.users.count_documents({"cards": block0[:10]}))
        else:
            try:
                # if authentication is not required check manufacturer code - 2 last digits of block0
                auth = block0[-2:] == config["manufacturer-code"]
            except KeyError:
                auth = False
        # add attempt to logs
        db.logs.insert(
            {
                "timestamp": dt.datetime.now(),
                "card": block0[:10],
                "manufacturer_code": block0[-2:],
                "auth_mode": "UID" if self.require_auth else "Manufacturer code",
                "success": auth,
            }
        )
        # send the authentication result over the interface
        write(f"AUTH {1 if auth else 0}")

    def check_auth(self):
        time = dt.datetime.now().time()
        try:
            # get the list of break times from database
            breaks = list(db.settings.find_one({"setting": "break_times"})["value"])
            # get the current setting
            require_auth = db.settings.find_one({"setting": "require_auth"})
            # if the current setting was set manually - don't check the time
            if bool(require_auth["manual"]):
                return bool(require_auth["value"])
            for item in breaks:
                # if current time is in one of the time ranges, return False, so auth is not required
                if time in DateTimeRange(item[0].time(), item[1].time()):
                    return False
            return True

        except (KeyError, TypeError):
            # default to requiring auth
            return True



from setuptools import setup, find_packages
from os import environ

__author__ = "opliko"
__license__ = "MIT"
__version__ = "0.5"
__status__ = "Prototype"

try:
    __version__ = environ["CHERRYDOOR_VERSION"]
except KeyError:
    pass
if "CI" in environ:
    with open("VERSION", "w", encoding="utf-8") as f:
        f.write(__version__)
with open("README.md", "r", encoding="utf-8") as f:
    readme = f.read()

setup(
    name="Cherrydoor",
    version=__version__,
    author="opliko",
    description=(
        "An overengineered rfid lock manager created for my school community. Made for Raspberry Pi connected with another microcontroler that send and recieved rfid data via UART"
    ),
    long_description=readme,
    long_description_content_type="text/markdown",
    license="MIT",
    keywords="wisniowasu mongodb flask rfid lock cherrydoor",
    url="https://github.com/oplik0/cherrydoor",
    packages=find_packages(),
    include_package_data=True,
    package_data={"": ["templates/*", "static/*/*", "static/images/*/*"]},
    zip_safe=False,
    scripts=["scripts/cherrydoor-install"],
    entry_points={"console_scripts": ["cherrydoor = cherrydoor.cherrydoor:cherrydoor"]},
    python_requires=">=3.7",
    install_requires=[
        "argon2-cffi>=19",
        "DateTimeRange>=0.6" "Flask>=1",
        "Flask-WTF>=0.14",
        "Flask-Login>=0.4",
        "Flask-PyMongo>=2.3",
        "Flask-RESTful>=0.3",
        "Flask-SocketIO>=4.2",
        "flask-talisman>=0.7",
        "pyserial>=3.4",
        "pymongo>=3.10",
        "eventlet",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3",
        "Topic :: Home Automation",
        "Topic :: Terminals :: Serial",
        "Topic :: Internet :: WWW/HTTP",
    ],
)
